# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['notebooks',
 'notebooks.bin',
 'notebooks.common',
 'notebooks.concept_co_occurrences',
 'notebooks.most_discriminating_words',
 'notebooks.named_entity_recognition',
 'notebooks.political_in_newspapers',
 'notebooks.political_in_newspapers.notebook_gui',
 'notebooks.political_in_newspapers.scripts',
 'notebooks.scrachpads',
 'notebooks.statens_offentliga_utredningar',
 'notebooks.textblock_politiskt',
 'notebooks.tfidf_word_siginificance',
 'notebooks.word_distribution_trends',
 'notebooks.word_trends']

package_data = \
{'': ['*']}

install_requires = \
['alive-progress>=1.6.1,<2.0.0',
 'appnope',
 'attrs',
 'beakerx==1.4.1',
 'bokeh',
 'click',
 'colorcet',
 'cython',
 'cytoolz',
 'decorator',
 'ftfy',
 'gensim',
 'holoviews',
 'humanfriendly',
 'humlab-penelope>=0.2.28,<0.3.0',
 'ipyaggrid',
 'ipycytoscape',
 'ipyfilechooser>=0.4.0,<0.5.0',
 'ipykernel',
 'ipympl',
 'ipywidgets',
 'joblib',
 'jupyter-bokeh',
 'jupyterlab',
 'lxml',
 'markdown',
 'matplotlib',
 'memoization',
 'more-itertools',
 'mypy>=0.782.0,<1.0.0',
 'networkx',
 'nltk',
 'numba',
 'numpy',
 'openpyxl',
 'pandas',
 'pandas-bokeh',
 'py4j',
 'pydotplus',
 'pyemd',
 'python-dotenv',
 'python-louvain',
 'qgrid>=1.3.1,<2.0.0',
 'scattertext',
 'scikit-learn',
 'scipy',
 'seaborn',
 'sklearn',
 'spacy==2.3.2',
 'statsmodels',
 'textacy==0.10.1',
 'tqdm',
 'traitlets==4.3.3',
 'wordcloud',
 'xlrd',
 'xmltodict']

setup_kwargs = {
    'name': 'humlab-westac',
    'version': '0.2.2',
    'description': 'Welfare State Analytics',
    'long_description': '# The Welfare State Analytics Text Analysis Repository\n\n## About the Project\n\nWelfare State Analytics. Text Mining and Modeling Swedish Politics, Media & Culture, 1945-1989 (WeStAc) is a digital humanities research project with five co-operatings partners: Umeå University, Uppsala University, Aalto University (Finland) and the National Library of Sweden.\n\nThe project will digitise literature, curate already digitised collections, and perform research via probabilistic methods and text mining models. WeStAc will both digitise and curate three massive textual datasets—in all, Big Data of almost four billion tokens—from the domains of Swedish politics, news media and literary culture during the second half of the 20th century.\n\n## Installation\n\n### Local install using pipenv\n\nSee [this page](https://github.com/humlab/welfare_state_analytics/wiki/How-to:-Install-notebooks-on-local-machine).\n\n### JupyterHub installation\n\nThe `westac_hub` repository contains a ready-to-use Docker setup (`Dockerfile` and `docker-compose.yml`) for a Jupyter Hub using `nginx` as reverse-proxy. The default setup uses `DockerSpawner` that spawns containers as specified in `westac_lab`, and Github for autorization (OAuth2). See the Makefile on how to build the project.\n\n### Single Docker container\n\nYou can also run the `westac_lab` container as a single Docker container if you have Docker installed on your computer.\n',
    'author': 'Roger Mähler',
    'author_email': 'roger.mahler@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://westac.se',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '==3.8.5',
}


setup(**setup_kwargs)
